@@ -1,5 +1,5 @@
 /* Unicode Normalization Form KD.
-   Copyright (C) 2009-2018 Free Software Foundation, Inc.
+   Copyright (C) 2009-2020 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2009.
 
    This program is free software: you can redistribute it and/or modify it
@@ -21,7 +21,7 @@
 #include "uninorm.h"
 
 #include "normalize-internal.h"
-#include "decompose-internal.h"
+#include "uninorm/decompose-internal.h"
 
 const struct unicode_normalization_form uninorm_nfkd =
   {
