@@ -175,7 +175,7 @@ value_to_data (const union value *value_, int width)
   if (width == 0)
     return &value->f;
   else
-    return value_str_rw (value, width);
+    return value->s;
 }
 
 /* Returns the number of bytes needed to store all the values in
@@ -604,8 +604,8 @@ datasheet_get_value (const struct datasheet *ds, casenumber row,
    successful, false on I/O error.  On failure, ROW might be
    partially modified or corrupted. */
 bool
-datasheet_put_value (struct datasheet *ds UNUSED, casenumber row UNUSED,
-                     size_t column UNUSED, const union value *value UNUSED)
+datasheet_put_value (struct datasheet *ds, casenumber row,
+                     size_t column, const union value *value)
 {
   return rw_case (ds, OP_WRITE, row, column, 1, (union value *) value);
 }
@@ -819,7 +819,7 @@ rw_case (struct datasheet *ds, enum rw_op op,
   assert (start_column + n_columns <= datasheet_get_n_columns (ds));
 
   prow = axis_map (ds->rows, lrow);
-  for (i = 0; i < n_columns; )
+  for (i = 0; i < n_columns;)
     {
       struct source *source = columns[i].source;
       size_t j;
