@@ -17,6 +17,7 @@
 #ifndef FILE_HANDLE_DEF_H
 #define FILE_HANDLE_DEF_H
 
+#include "libpspp/compiler.h"
 #include <stdbool.h>
 #include <stddef.h>
 
@@ -75,19 +76,18 @@ void fh_done (void);
 
 /* Creating file handles. */
 struct file_handle *fh_create_file (const char *handle_name,
-                                    const char *file_name,
+                                    const char *file_name, const char *file_name_encoding,
                                     const struct fh_properties *);
 struct file_handle *fh_create_dataset (struct dataset *);
 const struct fh_properties *fh_default_properties (void);
 
 /* Reference management. */
-struct file_handle *fh_ref (struct file_handle *);
+struct file_handle *fh_ref (struct file_handle *) WARN_UNUSED_RESULT;
 void fh_unref (struct file_handle *);
 void fh_unname (struct file_handle *);
 
 /* Finding file handles. */
 struct file_handle *fh_from_id (const char *handle_name);
-struct file_handle *fh_from_file_name (const char *file_name);
 struct file_handle *fh_inline_file (void);
 
 /* Generic properties of file handles. */
@@ -98,6 +98,7 @@ const char *fh_get_encoding (const struct file_handle *);
 
 /* Properties of FH_REF_FILE file handles. */
 const char *fh_get_file_name (const struct file_handle *);
+const char *fh_get_file_name_encoding (const struct file_handle *handle);
 enum fh_mode fh_get_mode (const struct file_handle *) ;
 enum fh_line_ends fh_get_line_ends (const struct file_handle *);
 
