@@ -76,8 +76,8 @@ struct settings
 
   enum settings_output_devices output_routing[SETTINGS_N_OUTPUT_TYPES];
 
-  enum settings_var_style var_output_style;
-  enum settings_value_style value_output_style;
+  enum settings_value_show show_values;
+  enum settings_value_show show_variables;
 };
 
 static struct settings the_settings = {
@@ -121,8 +121,8 @@ static struct settings the_settings = {
    0,
    SETTINGS_DEVICE_LISTING | SETTINGS_DEVICE_TERMINAL},
 
-  SETTINGS_VAR_STYLE_LABELS,
-  SETTINGS_VAL_STYLE_LABELS
+  SETTINGS_VALUE_SHOW_LABEL,
+  SETTINGS_VALUE_SHOW_LABEL
 };
 
 /* Initializes the settings module. */
@@ -190,7 +190,7 @@ settings_get_input_float_format (void)
 /* Sets the floating-point format used for RB and RBHEX input to
    FORMAT. */
 void
-settings_set_input_float_format ( enum float_format format)
+settings_set_input_float_format (enum float_format format)
 {
   the_settings.input_float_format = format;
 }
@@ -205,7 +205,7 @@ settings_get_input_integer_format (void)
 /* Sets the integer format used for IB and PIB input to
    FORMAT. */
 void
-settings_set_input_integer_format ( enum integer_format format)
+settings_set_input_integer_format (enum integer_format format)
 {
   the_settings.input_integer_format = format;
 }
@@ -234,7 +234,7 @@ settings_get_output_float_format (void)
 
 /* Sets the output float format to FLOAT_FORMAT. */
 void
-settings_set_output_float_format ( enum float_format float_format)
+settings_set_output_float_format (enum float_format float_format)
 {
   the_settings.output_float_format = float_format;
 }
@@ -248,7 +248,7 @@ settings_get_viewlength (void)
 
 /* Sets the view length. */
 void
-settings_set_viewlength ( int viewlength_)
+settings_set_viewlength (int viewlength_)
 {
   the_settings.viewlength = viewlength_;
 }
@@ -262,7 +262,7 @@ settings_get_viewwidth(void)
 
 /* Sets the screen width. */
 void
-settings_set_viewwidth ( int viewwidth_)
+settings_set_viewwidth (int viewwidth_)
 {
   the_settings.viewwidth = viewwidth_;
 }
@@ -290,7 +290,7 @@ settings_get_include (void)
 
 /* Set include file echo. */
 void
-settings_set_include ( bool include)
+settings_set_include (bool include)
 {
   the_settings.include = include;
 }
@@ -306,7 +306,7 @@ settings_get_epoch (void)
 
 /* Sets the year that starts the epoch. */
 void
-settings_set_epoch ( int epoch)
+settings_set_epoch (int epoch)
 {
   if (epoch < 0)
     {
@@ -328,7 +328,7 @@ settings_get_scompression (void)
 
 /* Set system file default compression. */
 void
-settings_set_scompression ( bool scompress)
+settings_set_scompression (bool scompress)
 {
   the_settings.scompress = scompress;
 }
@@ -342,7 +342,7 @@ settings_get_undefined (void)
 
 /* Set whether to warn on undefined values. */
 void
-settings_set_undefined ( bool undefined)
+settings_set_undefined (bool undefined)
 {
   the_settings.undefined = undefined;
 }
@@ -357,7 +357,7 @@ settings_get_blanks (void)
 /* Set the value that blank numeric fields are set to when read
    in. */
 void
-settings_set_blanks ( double blanks)
+settings_set_blanks (double blanks)
 {
   the_settings.blanks = blanks;
 }
@@ -385,13 +385,13 @@ settings_set_max_messages (enum msg_severity severity, int max)
 
   if (severity == MSG_S_WARNING)
     {
-      if ( max == 0)
+      if (max == 0)
 	{
 	  msg (MW,
 	       _("MXWARNS set to zero.  No further warnings will be given even when potentially problematic situations are encountered."));
 	  msg_ui_disable_warnings (true);
 	}
-      else if ( the_settings.max_messages [MSG_S_WARNING] == 0)
+      else if (the_settings.max_messages [MSG_S_WARNING] == 0)
 	{
 	  msg_ui_disable_warnings (false);
 	  the_settings.max_messages[MSG_S_WARNING] = max;
@@ -413,7 +413,7 @@ settings_get_mprint (void)
 /* Sets whether the commands generated by macro invocations are
    displayed. */
 void
-settings_set_mprint ( bool mprint)
+settings_set_mprint (bool mprint)
 {
   the_settings.mprint = mprint;
 }
@@ -427,7 +427,7 @@ settings_get_mxloops (void)
 
 /* Set implied limit of unbounded loop. */
 void
-settings_set_mxloops ( int mxloops)
+settings_set_mxloops (int mxloops)
 {
   the_settings.mxloops = mxloops;
 }
@@ -469,7 +469,7 @@ settings_get_format (void)
 /* Set default format for variables created by transformations
    and by DATA LIST {FREE,LIST}. */
 void
-settings_set_format ( const struct fmt_spec *default_format)
+settings_set_format (const struct fmt_spec *default_format)
 {
   the_settings.default_format = *default_format;
 }
@@ -484,7 +484,7 @@ settings_get_testing_mode (void)
 
 /* Set testing mode. */
 void
-settings_set_testing_mode ( bool testing_mode)
+settings_set_testing_mode (bool testing_mode)
 {
   the_settings.testing_mode = testing_mode;
 }
@@ -517,7 +517,7 @@ settings_set_algorithm (enum behavior_mode mode)
 
 /* Set the algorithm option for this command only */
 void
-settings_set_cmd_algorithm ( enum behavior_mode mode)
+settings_set_cmd_algorithm (enum behavior_mode mode)
 {
   the_settings.cmd_algorithm = mode;
 }
@@ -538,7 +538,7 @@ settings_get_syntax (void)
 
 /* Set the syntax option */
 void
-settings_set_syntax ( enum behavior_mode mode)
+settings_set_syntax (enum behavior_mode mode)
 {
   the_settings.syntax = mode;
 }
@@ -674,7 +674,7 @@ settings_dollar_template (const struct fmt_spec *fmt)
   fns = fmt_settings_get_style (the_settings.styles, fmt->type);
 
   ds_put_byte (&str, '$');
-  for (c = MAX (fmt->w - fmt->d - 1, 0); c > 0; )
+  for (c = MAX (fmt->w - fmt->d - 1, 0); c > 0;)
     {
       ds_put_byte (&str, '#');
       if (--c % 4 == 0 && c > 0)
@@ -707,29 +707,27 @@ settings_get_output_routing (enum settings_output_type type)
   return the_settings.output_routing[type] | SETTINGS_DEVICE_UNFILTERED;
 }
 
-enum settings_value_style
-settings_get_value_style (void)
+enum settings_value_show
+settings_get_show_values (void)
 {
-  return the_settings.value_output_style;
+  return the_settings.show_values;
 }
 
 void
-settings_set_value_style (enum settings_value_style s)
+settings_set_show_values (enum settings_value_show s)
 {
-  the_settings.value_output_style = s;
+  the_settings.show_values = s;
 }
 
 
-
-enum settings_var_style
-settings_get_var_style (void)
+enum settings_value_show
+settings_get_show_variables (void)
 {
-  return the_settings.var_output_style;
+  return the_settings.show_variables;
 }
 
-
 void
-settings_set_var_style (enum settings_var_style s)
+settings_set_show_variables (enum settings_value_show s)
 {
-  the_settings.var_output_style = s;
+  the_settings.show_variables = s;
 }
