@@ -36,14 +36,14 @@ void settings_set (const struct settings *);
 void settings_destroy (struct settings *);
 
 enum float_format settings_get_input_float_format (void);
-void settings_set_input_float_format ( enum float_format);
+void settings_set_input_float_format (enum float_format);
 
 /* Returns the integer format used for IB and PIB input. */
 enum integer_format settings_get_input_integer_format (void);
 
 /* Sets the integer format used for IB and PIB input to
    FORMAT. */
-void settings_set_input_integer_format ( enum integer_format);
+void settings_set_input_integer_format (enum integer_format);
 
 
 /* Returns the current output integer format. */
@@ -61,19 +61,19 @@ void settings_set_output_float_format (enum float_format float_format);
 
 
 int settings_get_viewlength (void);
-void settings_set_viewlength ( int);
+void settings_set_viewlength (int);
 
 int settings_get_viewwidth (void);
-void settings_set_viewwidth ( int);
+void settings_set_viewwidth (int);
 
 bool settings_get_safer_mode (void);
 void settings_set_safer_mode (void);
 
 bool settings_get_include (void);
-void settings_set_include ( bool);
+void settings_set_include (bool);
 
 int settings_get_epoch (void);
-void settings_set_epoch ( int);
+void settings_set_epoch (int);
 
 bool settings_get_scompression (void);
 void settings_set_scompression (bool);
@@ -90,14 +90,14 @@ bool settings_get_mprint (void);
 void settings_set_mprint (bool);
 
 int settings_get_mxloops (void);
-void settings_set_mxloops ( int);
+void settings_set_mxloops (int);
 
 size_t settings_get_workspace (void);
 size_t settings_get_workspace_cases (const struct caseproto *);
 void settings_set_workspace (size_t);
 
 const struct fmt_spec *settings_get_format (void);
-void settings_set_format ( const struct fmt_spec *);
+void settings_set_format (const struct fmt_spec *);
 
 bool settings_get_testing_mode (void);
 void settings_set_testing_mode (bool);
@@ -105,26 +105,26 @@ void settings_set_testing_mode (bool);
 int settings_get_fuzzbits (void);
 void settings_set_fuzzbits (int);
 
-enum settings_var_style
+/* Whether to show variable or value labels or the underlying value or variable
+   name. */
+enum settings_value_show
   {
-    SETTINGS_VAR_STYLE_NAMES,
-    SETTINGS_VAR_STYLE_LABELS,
-    SETTINGS_VAR_STYLE_BOTH
+    /* Use higher-level default.
+       In a pivot_value, the default is taken from the pivot_table.
+       In a pivot_table, the default is a global default.
+       As a global default, this is invalid. */
+    SETTINGS_VALUE_SHOW_DEFAULT = 0,
+
+    SETTINGS_VALUE_SHOW_VALUE = 1, /* Show value or variable name only. */
+    SETTINGS_VALUE_SHOW_LABEL = 2, /* Show label only. */
+    SETTINGS_VALUE_SHOW_BOTH = 3,  /* Show both value/name and label. */
   };
 
-enum settings_value_style
-  {
-    SETTINGS_VAL_STYLE_VALUES,
-    SETTINGS_VAL_STYLE_LABELS,
-    SETTINGS_VAL_STYLE_BOTH
-  };
+enum settings_value_show settings_get_show_values (void);
+enum settings_value_show settings_get_show_variables (void);
 
-
-enum settings_value_style settings_get_value_style (void);
-enum settings_var_style settings_get_var_style (void);
-
-void settings_set_value_style (enum settings_value_style s);
-void settings_set_var_style (enum settings_var_style s);
+void settings_set_show_values (enum settings_value_show);
+void settings_set_show_variables (enum settings_value_show);
 
 
 enum behavior_mode {
