@@ -1,5 +1,6 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2006, 2009, 2010, 2011, 2012, 2013, 2014, 2016 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2006, 2009, 2010, 2011, 2012, 2013,
+   2014, 2016, 2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -82,6 +83,7 @@ const GEnumValue role[] =
 /* A variable. */
 struct variable
   {
+    int ref_cnt;
     /* Dictionary information. */
     char *name;                 /* Variable name.  Mixed case. */
     int width;			/* 0 for numeric, otherwise string width. */
@@ -145,28 +147,43 @@ var_create (const char *name, int width)
   attrset_init (&v->attributes);
   ds_init_empty (&v->name_and_label);
 
+  v->ref_cnt = 1;
+
   return v;
 }
 
 /* Destroys variable V.
    V must not belong to a dictionary.  If it does, use
    dict_delete_var instead. */
-void
-var_destroy (struct variable *v)
+static void
+var_destroy__ (struct variable *v)
 {
-  if (v != NULL)
-    {
-      assert (!var_has_vardict (v));
-      mv_destroy (&v->miss);
-      var_clear_short_names (v);
-      val_labs_destroy (v->val_labs);
-      var_set_label_quiet (v, NULL);
-      attrset_destroy (var_get_attributes (v));
-      free (v->name);
-      ds_destroy (&v->name_and_label);
-      free (v);
-    }
+  assert (!var_has_vardict (v));
+  mv_destroy (&v->miss);
+  var_clear_short_names (v);
+  val_labs_destroy (v->val_labs);
+  var_set_label_quiet (v, NULL);
+  attrset_destroy (var_get_attributes (v));
+  free (v->name);
+  ds_destroy (&v->name_and_label);
+  free (v);
 }
+
+struct variable *
+var_ref (struct variable *v)
+{
+  v->ref_cnt++;
+  return v;
+}
+
+void
+var_unref (struct variable *v)
+{
+  if (--v->ref_cnt == 0)
+    var_destroy__ (v);
+}
+
+
 
 /* Variable names. */
 
@@ -290,17 +307,15 @@ var_set_width_and_formats (struct variable *v, int new_width,
 
   ov = var_clone (v);
 
-  if (var_has_missing_values (v))
+  if (mv_is_resizable (&v->miss, new_width))
+    mv_resize (&v->miss, new_width);
+  else
     {
-      if (mv_is_resizable (&v->miss, new_width))
-	mv_resize (&v->miss, new_width);
-      else
-	{
-	  mv_destroy (&v->miss);
-	  mv_init (&v->miss, new_width);
-	}
-      traits |= VAR_TRAIT_MISSING_VALUES;
+      mv_destroy (&v->miss);
+      mv_init (&v->miss, new_width);
     }
+  if (new_width != var_get_width (v))
+    traits |= VAR_TRAIT_MISSING_VALUES;
 
   if (v->val_labs != NULL)
     {
@@ -558,10 +573,40 @@ append_value (const struct variable *v, const union value *value,
 	      struct string *str)
 {
   char *s = data_out (value, var_get_encoding (v), &v->print);
-  ds_put_cstr (str, s);
+  struct substring ss = ss_cstr (s);
+  ss_rtrim (&ss, ss_cstr (" "));
+  ds_put_substring (str, ss);
   free (s);
 }
 
+void
+var_append_value_name__ (const struct variable *v, const union value *value,
+                         enum settings_value_show show, struct string *str)
+{
+  const char *label = var_lookup_value_label (v, value);
+
+  switch (show)
+    {
+    case SETTINGS_VALUE_SHOW_VALUE:
+      append_value (v, value, str);
+      break;
+
+    default:
+    case SETTINGS_VALUE_SHOW_LABEL:
+      if (label)
+	ds_put_cstr (str, label);
+      else
+	append_value (v, value, str);
+      break;
+
+    case SETTINGS_VALUE_SHOW_BOTH:
+      append_value (v, value, str);
+      if (label != NULL)
+        ds_put_format (str, " %s", label);
+      break;
+    }
+}
+
 /* Append STR with a string representing VALUE for variable V.
    That is, if VALUE has a label, append that label,
    otherwise format VALUE and append the formatted string.
@@ -571,33 +616,7 @@ void
 var_append_value_name (const struct variable *v, const union value *value,
 		       struct string *str)
 {
-  enum settings_value_style style = settings_get_value_style ();
-  const char *name = var_lookup_value_label (v, value);
-
-  switch (style)
-    {
-    case SETTINGS_VAL_STYLE_VALUES:
-      append_value (v, value, str);
-      break;
-
-    case SETTINGS_VAL_STYLE_LABELS:
-      if (name == NULL)
-	append_value (v, value, str);
-      else
-	ds_put_cstr (str, name);
-      break;
-
-    case SETTINGS_VAL_STYLE_BOTH:
-    default:
-      append_value (v, value, str);
-      if (name != NULL)
-	{
-	  ds_put_cstr (str, " (");
-	  ds_put_cstr (str, name);
-	  ds_put_cstr (str, ")");
-	}
-      break;
-    };
+  var_append_value_name__ (v, value, settings_get_show_values (), str);
 }
 
 /* Print and write formats. */
@@ -720,24 +739,19 @@ update_vl_string (const struct variable *v)
 const char *
 var_to_string (const struct variable *v)
 {
-  enum settings_var_style style = settings_get_var_style ();
-
-  switch (style)
-  {
-    case SETTINGS_VAR_STYLE_NAMES:
+  switch (settings_get_show_variables ())
+    {
+    case SETTINGS_VALUE_SHOW_VALUE:
       return v->name;
-      break;
-    case SETTINGS_VAR_STYLE_LABELS:
+
+    case SETTINGS_VALUE_SHOW_LABEL:
+    default:
       return v->label != NULL ? v->label : v->name;
-      break;
-    case SETTINGS_VAR_STYLE_BOTH:
+
+    case SETTINGS_VALUE_SHOW_BOTH:
       update_vl_string (v);
       return ds_cstr (&v->name_and_label);
-      break;
-    default:
-      NOT_REACHED ();
-      break;
-  };
+    }
 }
 
 /* Returns V's variable label, or a null pointer if it has none. */
@@ -802,12 +816,13 @@ measure_is_valid (enum measure m)
   return m == MEASURE_NOMINAL || m == MEASURE_ORDINAL || m == MEASURE_SCALE;
 }
 
-/* Returns a string version of measurement level M, for display to a user. */
+/* Returns a string version of measurement level M, for display to a user.
+   The caller may translate the string by passing it to gettext(). */
 const char *
 measure_to_string (enum measure m)
 {
   assert (m == measure[m].value);
-  return gettext (measure[m].label);
+  return measure[m].label;
 }
 
 /* Returns a string version of measurement level M, for use in PSPP command
@@ -887,12 +902,13 @@ var_role_is_valid (enum var_role role)
     }
 }
 
-/* Returns a string version of ROLE, for display to a user. */
+/* Returns a string version of ROLE, for display to a user.
+   The caller may translate the string by passing it to gettext(). */
 const char *
 var_role_to_string (enum var_role r)
 {
   assert (r == role[r].value);
-  return gettext (role[r].label);
+  return role[r].label;
 }
 
 /* Returns a string version of ROLE, for use in PSPP comamnd syntax. */
@@ -994,12 +1010,13 @@ alignment_is_valid (enum alignment a)
   return a == ALIGN_LEFT || a == ALIGN_RIGHT || a == ALIGN_CENTRE;
 }
 
-/* Returns a string version of alignment A, for display to a user. */
+/* Returns a string version of alignment A, for display to a user.
+   The caller may translate the string by passing it to gettext(). */
 const char *
 alignment_to_string (enum alignment a)
 {
   assert (a == align[a].value);
-  return gettext (align[a].label);
+  return align[a].label;
 }
 
 /* Returns a string version of alignment A, for use in PSPP command syntax. */
