@@ -40,6 +40,7 @@ struct dataset;
 void binomial_execute (const struct dataset *,
 		       struct casereader *,
                        enum mv_class,
-		       const struct npar_test *);
+		       const struct npar_test *,
+		       bool, double);
 
 #endif
