@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2009, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,15 +18,14 @@
 
 #include <stdlib.h>
 
-#include <data/case.h>
-#include <data/procedure.h>
-#include <data/transformations.h>
-#include <language/command.h>
-#include <language/lexer/lexer.h>
+#include "data/case.h"
+#include "data/dataset.h"
+#include "data/transformations.h"
+#include "language/command.h"
+#include "language/lexer/lexer.h"
+#include "libpspp/message.h"
 
 static int trns_fail (void *x, struct ccase **c, casenumber n);
-
-
 
 /* A transformation which is guaranteed to fail. */
 
@@ -34,15 +33,13 @@ static int
 trns_fail (void *x UNUSED, struct ccase **c UNUSED,
 	   casenumber n UNUSED)
 {
+  msg (SE, "DEBUG XFORM FAIL transformation executed");
   return TRNS_ERROR;
 }
 
-
 int
-cmd_debug_xform_fail (struct lexer *lexer, struct dataset *ds)
+cmd_debug_xform_fail (struct lexer *lexer UNUSED, struct dataset *ds)
 {
-
   add_transformation (ds, trns_fail, NULL, NULL);
-
-  return lex_end_of_command (lexer);
+  return CMD_SUCCESS;
 }
