@@ -52,14 +52,11 @@ static int messages_disabled;
 void
 vmsg (enum msg_class class, const char *format, va_list args)
 {
-  struct msg m;
-
-  m.category = msg_class_to_category (class);
-  m.severity = msg_class_to_severity (class);
-  m.text = xvasprintf (format, args);
-  m.file_name = NULL;
-  m.first_line = m.last_line = 0;
-  m.first_column = m.last_column = 0;
+  struct msg m = {
+    .category = msg_class_to_category (class),
+    .severity = msg_class_to_severity (class),
+    .text = xvasprintf (format, args),
+  };
 
   msg_emit (&m);
 }
@@ -81,23 +78,19 @@ void
 msg_error (int errnum, const char *format, ...)
 {
   va_list args;
-  char *e;
-  struct msg m;
-
-  m.category = MSG_C_GENERAL;
-  m.severity = MSG_S_ERROR;
-
   va_start (args, format);
-  e = xvasprintf (format, args);
+  char *e = xvasprintf (format, args);
   va_end (args);
 
-  m.file_name = NULL;
-  m.first_line = m.last_line = 0;
-  m.first_column = m.last_column = 0;
-  m.text = xasprintf (_("%s: %s"), e, strerror (errnum));
-  free (e);
-
+  struct msg m = {
+    .category = MSG_C_GENERAL,
+    .severity = MSG_S_ERROR,
+    .file_name = NULL,
+    .text = xasprintf (_("%s: %s"), e, strerror (errnum)),
+  };
   msg_emit (&m);
+
+  free (e);
 }
 
 
@@ -135,6 +128,8 @@ msg_dup (const struct msg *m)
   new_msg = xmemdup (m, sizeof *m);
   if (m->file_name != NULL)
     new_msg->file_name = xstrdup (m->file_name);
+  if (m->command_name != NULL)
+    new_msg->command_name = xstrdup (m->command_name);
   new_msg->text = xstrdup (m->text);
 
   return new_msg;
@@ -150,11 +145,12 @@ msg_destroy (struct msg *m)
 {
   free (m->file_name);
   free (m->text);
+  free (m->command_name);
   free (m);
 }
 
 char *
-msg_to_string (const struct msg *m, const char *command_name)
+msg_to_string (const struct msg *m)
 {
   struct string s;
 
@@ -217,8 +213,8 @@ msg_to_string (const struct msg *m, const char *command_name)
 
   ds_put_format (&s, "%s: ", msg_severity_to_string (m->severity));
 
-  if (m->category == MSG_C_SYNTAX && command_name != NULL)
-    ds_put_format (&s, "%s: ", command_name);
+  if (m->category == MSG_C_SYNTAX && m->command_name != NULL)
+    ds_put_format (&s, "%s: ", m->command_name);
 
   ds_put_cstr (&s, m->text);
 
@@ -277,7 +273,7 @@ static void
 ship_message (struct msg *m)
 {
   entrances++;
-  if ( ! m->shipped )
+  if (! m->shipped)
     {
       if (msg_handler && entrances <= 1)
 	msg_handler (m, msg_aux);
@@ -294,18 +290,11 @@ ship_message (struct msg *m)
 static void
 submit_note (char *s)
 {
-  struct msg m;
-
-  m.category = MSG_C_GENERAL;
-  m.severity = MSG_S_NOTE;
-  m.file_name = NULL;
-  m.first_line = 0;
-  m.last_line = 0;
-  m.first_column = 0;
-  m.last_column = 0;
-  m.text = s;
-  m.shipped = false;
-
+  struct msg m = {
+    .category = MSG_C_GENERAL,
+    .severity = MSG_S_NOTE,
+    .text = s,
+  };
   ship_message (&m);
 
   free (s);
@@ -320,7 +309,7 @@ process_msg (struct msg *m)
 
   if (too_many_errors
       || (too_many_notes && m->severity == MSG_S_NOTE)
-      || (warnings_off && m->severity == MSG_S_WARNING) )
+      || (warnings_off && m->severity == MSG_S_WARNING))
     return;
 
   ship_message (m);
@@ -363,6 +352,7 @@ msg_emit (struct msg *m)
      process_msg (m);
 
   free (m->text);
+  free (m->command_name);
 }
 
 /* Disables message output until the next call to msg_enable.  If
@@ -406,7 +396,7 @@ request_bug_report (const char *msg)
 #else
            "Unknown"
 #endif
-           );
+);
   fprintf (stderr, "******************************************************\n");
 }
 
