@@ -72,7 +72,7 @@ struct covariance
 {
   /* True if the covariances are centerered. (ie Real covariances) */
   bool centered;
-  
+
   /* The variables for which the covariance matrix is to be calculated. */
   size_t n_vars;
   const struct variable *const *vars;
@@ -165,7 +165,7 @@ covariance_1pass_create (size_t n_vars, const struct variable *const *vars,
 
   cov->exclude = exclude;
 
-  cov->n_cm = (n_vars * (n_vars - 1)  ) / 2;
+  cov->n_cm = (n_vars * (n_vars - 1)) / 2;
 
 
   cov->cm = xcalloc (cov->n_cm, sizeof *cov->cm);
@@ -232,13 +232,13 @@ cm_idx (const struct covariance *cov, int i, int j)
   assert (i >= 0);
   assert (j < cov->dim);
 
-  if ( i == 0)
+  if (i == 0)
     return -1;
 
   if (j >= cov->dim - 1)
     return -1;
 
-  if ( i <= j)
+  if (i <= j)
     return -1 ;
 
   as = nj * (nj + 1) ;
@@ -269,7 +269,7 @@ is_missing (const struct covariance *cov, int i, const struct ccase *c)
 static double
 get_val (const struct covariance *cov, int i, const struct ccase *c)
 {
-  if ( i < cov->n_vars)
+  if (i < cov->n_vars)
     {
       const struct variable *var = cov->vars[i];
 
@@ -317,14 +317,14 @@ covariance_accumulate_pass1 (struct covariance *cov, const struct ccase *c)
     {
       double v1 = get_val (cov, i, c);
 
-      if ( is_missing (cov, i, c))
+      if (is_missing (cov, i, c))
 	continue;
 
       for (j = 0 ; j < cov->dim; ++j)
 	{
 	  double pwr = 1.0;
 
-	  if ( is_missing (cov, j, c))
+	  if (is_missing (cov, j, c))
 	    continue;
 
 	  for (m = 0 ; m <= MOMENT_MEAN; ++m)
@@ -365,7 +365,7 @@ covariance_accumulate_pass2 (struct covariance *cov, const struct ccase *c)
       if (cov->categoricals)
 	cov->dim += categoricals_df_total (cov->categoricals);
 
-      cov->n_cm = (cov->dim * (cov->dim - 1)  ) / 2;
+      cov->n_cm = (cov->dim * (cov->dim - 1)) / 2;
       cov->cm = xcalloc (cov->n_cm, sizeof *cov->cm);
 
       /* Grow the moment matrices so that they're large enough to accommodate the
@@ -419,7 +419,7 @@ covariance_accumulate_pass2 (struct covariance *cov, const struct ccase *c)
     {
       double v1 = get_val (cov, i, c);
 
-      if ( is_missing (cov, i, c))
+      if (is_missing (cov, i, c))
 	continue;
 
       for (j = 0 ; j < cov->dim; ++j)
@@ -430,7 +430,7 @@ covariance_accumulate_pass2 (struct covariance *cov, const struct ccase *c)
 
 	  const double s = pow2 (v1 - gsl_matrix_get (cov->moments[MOMENT_MEAN], i, j)) * weight;
 
-	  if ( is_missing (cov, j, c))
+	  if (is_missing (cov, j, c))
 	    continue;
 
 	  {
@@ -468,9 +468,9 @@ covariance_accumulate (struct covariance *cov, const struct ccase *c)
 
   assert (cov->passes == 1);
 
-  if ( !cov->pass_one_first_case_seen)
+  if (!cov->pass_one_first_case_seen)
     {
-      assert ( cov->state == 0);
+      assert (cov->state == 0);
       cov->state = 1;
     }
 
@@ -478,7 +478,7 @@ covariance_accumulate (struct covariance *cov, const struct ccase *c)
     {
       const union value *val1 = case_data (c, cov->vars[i]);
 
-      if ( is_missing (cov, i, c))
+      if (is_missing (cov, i, c))
 	continue;
 
       for (j = 0 ; j < cov->dim; ++j)
@@ -487,7 +487,7 @@ covariance_accumulate (struct covariance *cov, const struct ccase *c)
 	  int idx;
 	  const union value *val2 = case_data (c, cov->vars[j]);
 
-	  if ( is_missing (cov, j, c))
+	  if (is_missing (cov, j, c))
 	    continue;
 
 	  idx = cm_idx (cov, i, j);
@@ -521,7 +521,7 @@ cm_to_gsl (struct covariance *cov)
   gsl_matrix *m = gsl_matrix_calloc (cov->dim, cov->dim);
 
   /* Copy the non-diagonal elements from cov->cm */
-  for ( j = 0 ; j < cov->dim - 1; ++j)
+  for (j = 0 ; j < cov->dim - 1; ++j)
     {
       for (i = j+1 ; i < cov->dim; ++i)
 	{
@@ -555,7 +555,7 @@ covariance_calculate_double_pass (struct covariance *cov)
 	  *x /= gsl_matrix_get (cov->moments[MOMENT_NONE], i, j);
 
 	  idx = cm_idx (cov, i, j);
-	  if ( idx >= 0)
+	  if (idx >= 0)
 	    {
 	      x = &cov->cm [idx];
 	      *x /= gsl_matrix_get (cov->moments[MOMENT_NONE], i, j);
@@ -575,7 +575,7 @@ covariance_calculate_single_pass (struct covariance *cov)
   for (m = 0; m < n_MOMENTS; ++m)
     {
       /* Divide the moments by the number of samples */
-      if ( m > 0)
+      if (m > 0)
 	{
 	  for (i = 0 ; i < cov->dim; ++i)
 	    {
@@ -584,7 +584,7 @@ covariance_calculate_single_pass (struct covariance *cov)
 		  double *x = gsl_matrix_ptr (cov->moments[m], i, j);
 		  *x /= gsl_matrix_get (cov->moments[0], i, j);
 
-		  if ( m == MOMENT_VARIANCE)
+		  if (m == MOMENT_VARIANCE)
 		    *x -= pow2 (gsl_matrix_get (cov->moments[1], i, j));
 		}
 	    }
@@ -594,7 +594,7 @@ covariance_calculate_single_pass (struct covariance *cov)
   if (cov->centered)
     {
       /* Centre the moments */
-      for ( j = 0 ; j < cov->dim - 1; ++j)
+      for (j = 0 ; j < cov->dim - 1; ++j)
 	{
 	  for (i = j + 1 ; i < cov->dim; ++i)
 	    {
@@ -622,7 +622,7 @@ covariance_calculate_single_pass (struct covariance *cov)
 gsl_matrix *
 covariance_calculate (struct covariance *cov)
 {
-  if ( cov->state <= 0 )
+  if (cov->state <= 0)
     return NULL;
 
   switch (cov->passes)
@@ -664,7 +664,7 @@ covariance_calculate_single_pass_unnormalized (struct covariance *cov)
 	    }
 	}
 
-      for ( j = 0 ; j < cov->dim - 1; ++j)
+      for (j = 0 ; j < cov->dim - 1; ++j)
 	{
 	  for (i = j + 1 ; i < cov->dim; ++i)
 	    {
@@ -678,7 +678,7 @@ covariance_calculate_single_pass_unnormalized (struct covariance *cov)
 	    }
 	}
     }
-  
+
   return cm_to_gsl (cov);
 }
 
@@ -690,7 +690,7 @@ covariance_calculate_single_pass_unnormalized (struct covariance *cov)
 const gsl_matrix *
 covariance_calculate_unnormalized (struct covariance *cov)
 {
-  if ( cov->state <= 0 )
+  if (cov->state <= 0)
     return NULL;
 
   if (cov->unnormalised != NULL)
@@ -754,62 +754,46 @@ covariance_dim (const struct covariance * cov)
 */
 
 #include "libpspp/str.h"
-#include "output/tab.h"
+#include "output/pivot-table.h"
 #include "data/format.h"
 
 
 /* Create a table which can be populated with the encodings for
    the covariance matrix COV */
-struct tab_table *
-covariance_dump_enc_header (const struct covariance *cov, int length)
+struct pivot_table *
+covariance_dump_enc_header (const struct covariance *cov)
 {
-  struct tab_table *t = tab_create (cov->dim,  length);
-  int n;
-  int i;
+  struct pivot_table *table = pivot_table_create ("Covariance Encoding");
 
-  tab_title (t, "Covariance Encoding");
-
-  tab_box (t,
-	   TAL_2, TAL_2, 0, 0,
-	   0, 0,   tab_nc (t) - 1,   tab_nr (t) - 1);
-
-  tab_hline (t, TAL_2, 0, tab_nc (t) - 1, 1);
-
-
-  for (i = 0 ; i < cov->n_vars; ++i)
+  struct pivot_dimension *factors = pivot_dimension_create (
+    table, PIVOT_AXIS_COLUMN, "Factor");
+  for (size_t i = 0 ; i < cov->n_vars; ++i)
+    pivot_category_create_leaf (factors->root,
+                                pivot_value_new_variable (cov->vars[i]));
+  for (size_t i = 0, n = 0; i < cov->dim - cov->n_vars; n++)
     {
-      tab_text (t, i, 0, TAT_TITLE, var_get_name (cov->vars[i]));
-      tab_vline (t, TAL_1, i + 1, 0, tab_nr (t) - 1);
-    }
-
-  n = 0;
-  while (i < cov->dim)
-    {
-      struct string str;
-      int idx = i - cov->n_vars;
       const struct interaction *iact =
-	categoricals_get_interaction_by_subscript (cov->categoricals, idx);
-      int df;
+	categoricals_get_interaction_by_subscript (cov->categoricals, i);
 
-      ds_init_empty (&str);
+      struct string str = DS_EMPTY_INITIALIZER;
       interaction_to_string (iact, &str);
+      struct pivot_category *group = pivot_category_create_group__ (
+        factors->root,
+        pivot_value_new_user_text_nocopy (ds_steal_cstr (&str)));
 
-      df = categoricals_df (cov->categoricals, n);
-
-      tab_joint_text (t,
-		      i, 0,
-		      i + df - 1, 0,
-		      TAT_TITLE, ds_cstr (&str));
-
-      if (i + df < tab_nr (t) - 1)
-	tab_vline (t, TAL_1, i + df, 0, tab_nr (t) - 1);
+      int df = categoricals_df (cov->categoricals, n);
+      for (int j = 0; j < df; j++)
+        pivot_category_create_leaf_rc (group, pivot_value_new_integer (j),
+                                       PIVOT_RC_INTEGER);
 
       i += df;
-      n++;
-      ds_destroy (&str);
     }
 
-  return t;
+  struct pivot_dimension *matrix = pivot_dimension_create (
+    table, PIVOT_AXIS_ROW, "Matrix", "Matrix");
+  matrix->hide_all_labels = true;
+
+  return table;
 }
 
 
@@ -819,14 +803,13 @@ covariance_dump_enc_header (const struct covariance *cov, int length)
  */
 void
 covariance_dump_enc (const struct covariance *cov, const struct ccase *c,
-		     struct tab_table *t)
+		     struct pivot_table *table)
 {
-  static int row = 0;
-  int i;
-  ++row;
-  for (i = 0 ; i < cov->dim; ++i)
-    {
-      double v = get_val (cov, i, c);
-      tab_double (t, i, row, 0, v, i < cov->n_vars ? NULL : &F_8_0, RC_OTHER);
-    }
+  int row = pivot_category_create_leaf (
+    table->dimensions[1]->root,
+    pivot_value_new_integer (table->dimensions[1]->n_leaves));
+
+  for (int i = 0 ; i < cov->dim; ++i)
+    pivot_table_put2 (
+      table, i, row, pivot_value_new_number (get_val (cov, i, c)));
 }
