@@ -33,6 +33,7 @@
 #include "output/table-item.h"
 #include "output/table-provider.h"
 
+#include "gl/minmax.h"
 #include "gl/xalloc.h"
 #include "gl/xvasprintf.h"
 
@@ -51,7 +52,6 @@ struct csv_driver
     bool captions;              /* Print table captions? */
 
     struct file_handle *handle;
-    char *command_name;         /* Current command. */
     FILE *file;                 /* Output file. */
     int n_items;                /* Number of items output so far. */
   };
@@ -127,128 +127,30 @@ csv_flush (struct output_driver *driver)
     fflush (csv->file);
 }
 
+static void
+csv_output_field__ (struct csv_driver *csv, struct substring field)
+{
+  ss_ltrim (&field, ss_cstr (" "));
+
+  if (csv->quote && ss_cspan (field, ss_cstr (csv->quote_set)) < field.length)
+    {
+      putc (csv->quote, csv->file);
+      for (size_t i = 0; i < field.length; i++)
+        {
+          if (field.string[i] == csv->quote)
+            putc (csv->quote, csv->file);
+          putc (field.string[i], csv->file);
+        }
+      putc (csv->quote, csv->file);
+    }
+  else
+    fwrite (field.string, field.length, 1, csv->file);
+}
+
 static void
 csv_output_field (struct csv_driver *csv, const char *field)
 {
-  while (*field == ' ')
-    field++;
-
-  if (csv->quote && field[strcspn (field, csv->quote_set)])
-    {
-      const char *p;
-
-      putc (csv->quote, csv->file);
-      for (p = field; *p != '\0'; p++)
-        {
-          if (*p == csv->quote)
-            putc (csv->quote, csv->file);
-          putc (*p, csv->file);
-        }
-      putc (csv->quote, csv->file);
-    }
-  else
-    fputs (field, csv->file);
-}
-
-static void PRINTF_FORMAT (2, 3)
-csv_output_field_format (struct csv_driver *csv, const char *format, ...)
-{
-  va_list args;
-  char *s;
-
-  va_start (args, format);
-  s = xvasprintf (format, args);
-  va_end (args);
-
-  csv_output_field (csv, s);
-  free (s);
-}
-
-static void
-csv_put_field (struct csv_driver *csv, struct string *s, const char *field)
-{
-  while (*field == ' ')
-    field++;
-
-  if (csv->quote && field[strcspn (field, csv->quote_set)])
-    {
-      const char *p;
-
-      ds_put_byte (s, csv->quote);
-      for (p = field; *p != '\0'; p++)
-        {
-          if (*p == csv->quote)
-            ds_put_byte (s, csv->quote);
-          ds_put_byte (s, *p);
-        }
-      ds_put_byte (s, csv->quote);
-    }
-  else
-    ds_put_cstr (s, field);
-}
-
-static void
-csv_output_subtable (struct csv_driver *csv, struct string *s,
-                     const struct table_item *item)
-{
-  const struct table *t = table_item_get_table (item);
-  const char *title = table_item_get_title (item);
-  const char *caption = table_item_get_caption (item);
-  int y, x;
-
-  if (csv->titles && title != NULL)
-    {
-      csv_output_field_format (csv, "Table: %s", title);
-      putc ('\n', csv->file);
-    }
-
-  for (y = 0; y < table_nr (t); y++)
-    {
-      if (y > 0)
-        ds_put_byte (s, '\n');
-
-      for (x = 0; x < table_nc (t); x++)
-        {
-          struct table_cell cell;
-
-          table_get_cell (t, x, y, &cell);
-
-          if (x > 0)
-            ds_put_cstr (s, csv->separator);
-
-          if (x != cell.d[TABLE_HORZ][0] || y != cell.d[TABLE_VERT][0])
-            csv_put_field (csv, s, "");
-          else if (cell.n_contents == 1 && cell.contents[0].text != NULL)
-            csv_put_field (csv, s, cell.contents[0].text);
-          else
-            {
-              struct string s2;
-              size_t i;
-
-              ds_init_empty (&s2);
-              for (i = 0; i < cell.n_contents; i++)
-                {
-                  if (i > 0)
-                    ds_put_cstr (&s2, "\n\n");
-
-                  if (cell.contents[i].text != NULL)
-                    ds_put_cstr (&s2, cell.contents[i].text);
-                  else
-                    csv_output_subtable (csv, &s2, cell.contents[i].table);
-                }
-              csv_put_field (csv, s, ds_cstr (&s2));
-              ds_destroy (&s2);
-            }
-
-          table_cell_free (&cell);
-        }
-    }
-
-  if (csv->captions && caption != NULL)
-    {
-      csv_output_field_format (csv, "Caption: %s", caption);
-      putc ('\n', csv->file);
-    }
+  csv_output_field__ (csv, ss_cstr (field));
 }
 
 static void
@@ -258,32 +160,60 @@ csv_put_separator (struct csv_driver *csv)
     putc ('\n', csv->file);
 }
 
+static void
+csv_output_lines (struct csv_driver *csv, const char *text_)
+{
+  struct substring text = ss_cstr (text_);
+  struct substring line;
+  size_t save_idx = 0;
+  while (ss_separate (text, ss_cstr ("\n"), &save_idx, &line))
+    {
+      csv_output_field__ (csv, line);
+      putc ('\n', csv->file);
+    }
+}
+
+static void
+csv_format_footnotes (const struct footnote **f, size_t n, struct string *s)
+{
+  for (size_t i = 0; i < n; i++)
+    ds_put_format (s, "[%s]", f[i]->marker);
+}
+
+static void
+csv_output_table_item_text (struct csv_driver *csv,
+                            const struct table_item_text *text,
+                            const char *leader)
+{
+  if (!text)
+    return;
+
+  struct string s = DS_EMPTY_INITIALIZER;
+  ds_put_format (&s, "%s: %s", leader, text->content);
+  csv_format_footnotes (text->footnotes, text->n_footnotes, &s);
+  csv_output_field (csv, ds_cstr (&s));
+  ds_destroy (&s);
+  putc ('\n', csv->file);
+}
+
 static void
 csv_submit (struct output_driver *driver,
             const struct output_item *output_item)
 {
   struct csv_driver *csv = csv_driver_cast (driver);
 
-  output_driver_track_current_command (output_item, &csv->command_name);
-
   if (is_table_item (output_item))
     {
       struct table_item *table_item = to_table_item (output_item);
-      const char *title = table_item_get_title (table_item);
-      const char *caption = table_item_get_caption (table_item);
       const struct table *t = table_item_get_table (table_item);
-      int footnote_idx;
       int x, y;
 
       csv_put_separator (csv);
 
-      if (csv->titles && title != NULL)
-        {
-          csv_output_field_format (csv, "Table: %s", title);
-          putc ('\n', csv->file);
-        }
+      if (csv->titles)
+        csv_output_table_item_text (csv, table_item_get_title (table_item),
+                                    "Table");
 
-      footnote_idx = 0;
       for (y = 0; y < table_nr (t); y++)
         {
           for (x = 0; x < table_nc (t); x++)
@@ -297,88 +227,55 @@ csv_submit (struct output_driver *driver,
 
               if (x != cell.d[TABLE_HORZ][0] || y != cell.d[TABLE_VERT][0])
                 csv_output_field (csv, "");
-              else if (cell.n_contents == 1
-                       && cell.contents[0].text != NULL
-                       && cell.contents[0].n_footnotes == 0)
-                csv_output_field (csv, cell.contents[0].text);
+              else if (!(cell.options & TAB_MARKUP) && !cell.n_footnotes
+                       && !cell.n_subscripts && !cell.superscript)
+                csv_output_field (csv, cell.text);
               else
                 {
-                  struct string s;
-                  size_t i;
+                  struct string s = DS_EMPTY_INITIALIZER;
 
-                  ds_init_empty (&s);
-                  for (i = 0; i < cell.n_contents; i++)
+                  if (cell.options & TAB_MARKUP)
                     {
-                      const struct cell_contents *c = &cell.contents[i];
-                      int j;
-
-                      if (i > 0)
-                        ds_put_cstr (&s, "\n\n");
-
-                      if (c->text != NULL)
-                        ds_put_cstr (&s, c->text);
-                      else
-                        csv_output_subtable (csv, &s, c->table);
-
-                      for (j = 0; j < c->n_footnotes; j++)
-                        {
-                          char marker[16];
-
-                          str_format_26adic (++footnote_idx, false,
-                                             marker, sizeof marker);
-                          ds_put_format (&s, "[%s]", marker);
-                        }
+                      char *t = output_get_text_from_markup (cell.text);
+                      ds_put_cstr (&s, t);
+                      free (t);
                     }
+                  else
+                    ds_put_cstr (&s, cell.text);
+
+                  if (cell.n_subscripts)
+                    for (size_t i = 0; i < cell.n_subscripts; i++)
+                      ds_put_format (&s, "%c%s",
+                                     i ? ',' : '_', cell.subscripts[i]);
+                  if (cell.superscript)
+                    ds_put_format (&s, "^%s", cell.superscript);
+                  csv_format_footnotes (cell.footnotes, cell.n_footnotes, &s);
                   csv_output_field (csv, ds_cstr (&s));
                   ds_destroy (&s);
                 }
-
-              table_cell_free (&cell);
             }
           putc ('\n', csv->file);
         }
 
-      if (csv->captions && caption != NULL)
-        {
-          csv_output_field_format (csv, "Caption: %s", caption);
-          putc ('\n', csv->file);
-        }
+      if (csv->captions)
+        csv_output_table_item_text (csv, table_item_get_caption (table_item),
+                                    "Caption");
 
-      if (footnote_idx)
+      const struct footnote **f;
+      size_t n_footnotes = table_collect_footnotes (table_item, &f);
+      if (n_footnotes)
         {
-          size_t i;
-
           fputs ("\nFootnotes:\n", csv->file);
 
-          footnote_idx = 0;
-          for (y = 0; y < table_nr (t); y++)
+          for (size_t i = 0; i < n_footnotes; i++)
             {
-              struct table_cell cell;
-              for (x = 0; x < table_nc (t); x = cell.d[TABLE_HORZ][1])
-                {
-                  table_get_cell (t, x, y, &cell);
-
-                  if (x == cell.d[TABLE_HORZ][0] && y == cell.d[TABLE_VERT][0])
-                    for (i = 0; i < cell.n_contents; i++)
-                      {
-                        const struct cell_contents *c = &cell.contents[i];
-                        int j;
-
-                        for (j = 0; j < c->n_footnotes; j++)
-                          {
-                            char marker[16];
-
-                            str_format_26adic (++footnote_idx, false,
-                                               marker, sizeof marker);
-                            csv_output_field (csv, marker);
-                            fputs (csv->separator, csv->file);
-                            csv_output_field (csv, c->footnotes[j]);
-                            putc ('\n', csv->file);
-                          }
-                      }
-                  table_cell_free (&cell);
-                }
+              csv_output_field (csv, f[i]->marker);
+              fputs (csv->separator, csv->file);
+              csv_output_field (csv, f[i]->content);
+              putc ('\n', csv->file);
             }
+
+          free (f);
         }
     }
   else if (is_text_item (output_item))
@@ -387,32 +284,24 @@ csv_submit (struct output_driver *driver,
       enum text_item_type type = text_item_get_type (text_item);
       const char *text = text_item_get_text (text_item);
 
-      if (type == TEXT_ITEM_COMMAND_OPEN || type == TEXT_ITEM_COMMAND_CLOSE
-          || type == TEXT_ITEM_SYNTAX)
+      if (type == TEXT_ITEM_SYNTAX || type == TEXT_ITEM_PAGE_TITLE)
         return;
 
       csv_put_separator (csv);
-      switch (type)
+
+      if (text_item->markup)
         {
-        case TEXT_ITEM_TITLE:
-          csv_output_field_format (csv, "Title: %s", text);
-          break;
-
-        case TEXT_ITEM_SUBTITLE:
-          csv_output_field_format (csv, "Subtitle: %s", text);
-          break;
-
-        default:
-          csv_output_field (csv, text);
-          break;
+          char *plain_text = output_get_text_from_markup (text);
+          csv_output_lines (csv, plain_text);
+          free (plain_text);
         }
-      putc ('\n', csv->file);
+      else
+        csv_output_lines (csv, text);
     }
   else if (is_message_item (output_item))
     {
       const struct message_item *message_item = to_message_item (output_item);
-      const struct msg *msg = message_item_get_msg (message_item);
-      char *s = msg_to_string (msg, csv->command_name);
+      char *s = msg_to_string (message_item_get_msg (message_item));
       csv_put_separator (csv);
       csv_output_field (csv, s);
       free (s);
