@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2007, 2009, 2010, 2011, 2012, 2014 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2007, 2009, 2010, 2011, 2012, 2014, 2019 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,23 +21,29 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <libxml/parser.h>
+#include <libxml/tree.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #include "data/file-handle-def.h"
 #include "data/settings.h"
 #include "libpspp/array.h"
 #include "libpspp/assertion.h"
+#include "libpspp/i18n.h"
 #include "libpspp/message.h"
 #include "libpspp/llx.h"
 #include "libpspp/string-map.h"
 #include "libpspp/string-set.h"
 #include "libpspp/str.h"
+#include "output/group-item.h"
 #include "output/message-item.h"
 #include "output/output-item.h"
 #include "output/text-item.h"
 
+#include "gl/error.h"
 #include "gl/xalloc.h"
 #include "gl/xmemdup0.h"
 
@@ -46,50 +52,87 @@
 
 struct output_engine
   {
+    struct ll ll;                  /* Node for this engine. */
     struct llx_list drivers;       /* Contains "struct output_driver"s. */
-    struct string deferred_syntax; /* TEXT_ITEM_SYNTAX being accumulated. */
+    struct string deferred_text;   /* Output text being accumulated. */
+    enum text_item_type deferred_type; /* Type of text being accumulated. */
+    char *command_name;            /* Name of command being processed. */
+    char *title, *subtitle;        /* Components of page title. */
+
+    /* Output grouping stack.
+
+       TEXT_ITEM_GROUP_OPEN pushes a group on the stack and
+       TEXT_ITEM_GROUP_CLOSE pops one off. */
+    char **groups;               /* Command names of nested sections. */
+    size_t n_groups;
+    size_t allocated_groups;
+
+    struct string_map heading_vars;
   };
 
-static const struct output_driver_factory *factories[];
+static struct ll_list engine_stack = LL_INITIALIZER (engine_stack);
 
-/* A stack of output engines.. */
-static struct output_engine *engine_stack;
-static size_t n_stack, allocated_stack;
+static const struct output_driver_factory *factories[];
 
 static struct output_engine *
 engine_stack_top (void)
 {
-  assert (n_stack > 0);
-  return &engine_stack[n_stack - 1];
+  struct ll *head = ll_head (&engine_stack);
+  if (ll_is_empty (&engine_stack))
+    return NULL;
+  return ll_data (head, struct output_engine, ll);
+}
+
+static void
+put_strftime (const char *key, const char *format,
+              const struct tm *tm, struct string_map *vars)
+{
+  if (!string_map_find (vars, key))
+    {
+      char value[128];
+      strftime (value, sizeof value, format, tm);
+      string_map_insert (vars, key, value);
+    }
 }
 
 void
 output_engine_push (void)
 {
-  struct output_engine *e;
+  struct output_engine *e = xzalloc (sizeof (*e));
 
-  if (n_stack >= allocated_stack)
-    engine_stack = x2nrealloc (engine_stack, &allocated_stack,
-                               sizeof *engine_stack);
-
-  e = &engine_stack[n_stack++];
   llx_init (&e->drivers);
-  ds_init_empty (&e->deferred_syntax);
+  ds_init_empty (&e->deferred_text);
+
+  string_map_init (&e->heading_vars);
+
+  time_t t = time (NULL);
+  const struct tm *tm = localtime (&t);
+  put_strftime ("Date", "%x", tm, &e->heading_vars);
+  put_strftime ("Time", "%X", tm, &e->heading_vars);
+
+  ll_push_head (&engine_stack, &e->ll);
 }
 
 void
 output_engine_pop (void)
 {
-  struct output_engine *e;
+  struct ll *head = ll_pop_head (&engine_stack);
+  struct output_engine *e =ll_data (head, struct output_engine, ll);
 
-  assert (n_stack > 0);
-  e = &engine_stack[--n_stack];
   while (!llx_is_empty (&e->drivers))
     {
       struct output_driver *d = llx_pop_head (&e->drivers, &llx_malloc_mgr);
       output_driver_destroy (d);
     }
-  ds_destroy (&e->deferred_syntax);
+  ds_destroy (&e->deferred_text);
+  free (e->command_name);
+  free (e->title);
+  free (e->subtitle);
+  for (size_t i = 0; i < e->n_groups; i++)
+    free (e->groups[i]);
+  free (e->groups);
+  string_map_destroy (&e->heading_vars);
+  free (e);
 }
 
 void
@@ -135,21 +178,43 @@ output_submit__ (struct output_engine *e, struct output_item *item)
 }
 
 static void
-flush_deferred_syntax (struct output_engine *e)
+flush_deferred_text (struct output_engine *e)
 {
-  if (!ds_is_empty (&e->deferred_syntax))
+  if (!ds_is_empty (&e->deferred_text))
     {
-      char *syntax = ds_steal_cstr (&e->deferred_syntax);
-      output_submit__ (e, text_item_super (
-                         text_item_create_nocopy (TEXT_ITEM_SYNTAX, syntax)));
+      char *text = ds_steal_cstr (&e->deferred_text);
+      output_submit__ (e, text_item_super (text_item_create_nocopy (
+                                             e->deferred_type, text)));
     }
 }
 
 static bool
-is_syntax_item (const struct output_item *item)
+defer_text (struct output_engine *e, struct output_item *item)
 {
-  return (is_text_item (item)
-          && text_item_get_type (to_text_item (item)) == TEXT_ITEM_SYNTAX);
+  if (!is_text_item (item))
+    return false;
+
+  struct text_item *text_item = to_text_item (item);
+  if (text_item->markup)        /* XXX */
+    return false;
+
+  enum text_item_type type = text_item_get_type (text_item);
+  if (type != TEXT_ITEM_SYNTAX && type != TEXT_ITEM_LOG)
+    return false;
+
+  if (!ds_is_empty (&e->deferred_text) && e->deferred_type != type)
+    flush_deferred_text (e);
+
+  e->deferred_type = type;
+
+  if (!ds_is_empty (&e->deferred_text))
+    ds_put_byte (&e->deferred_text, '\n');
+
+  const char *text = text_item_get_text (text_item);
+  ds_put_cstr (&e->deferred_text, text);
+  output_item_unref (item);
+
+  return true;
 }
 
 /* Submits ITEM to the configured output drivers, and transfers ownership to
@@ -159,20 +224,78 @@ output_submit (struct output_item *item)
 {
   struct output_engine *e = engine_stack_top ();
 
+  if (e == NULL)
+    return;
+
   if (item == NULL)
     return;
 
-  if (is_syntax_item (item))
+  if (defer_text (e, item))
+    return;
+  flush_deferred_text (e);
+
+  if (is_group_open_item (item))
     {
-      ds_put_cstr (&e->deferred_syntax, text_item_get_text (to_text_item (item)));
-      output_item_unref (item);
-      return;
+      const struct group_open_item *group_open_item
+        = to_group_open_item (item);
+      if (e->n_groups >= e->allocated_groups)
+        e->groups = x2nrealloc (e->groups, &e->allocated_groups,
+                                sizeof *e->groups);
+      e->groups[e->n_groups] = (group_open_item->command_name
+                                ? xstrdup (group_open_item->command_name)
+                                : NULL);
+      e->n_groups++;
+    }
+  else if (is_group_close_item (item))
+    {
+      assert (e->n_groups > 0);
+
+      size_t idx = --e->n_groups;
+      free (e->groups[idx]);
+      if (idx >= 1 && idx <= 4)
+        {
+          char *key = xasprintf ("Head%zu", idx);
+          free (string_map_find_and_delete (&e->heading_vars, key));
+          free (key);
+        }
+    }
+  else if (is_text_item (item))
+    {
+      const struct text_item *text_item = to_text_item (item);
+      enum text_item_type type = text_item_get_type (text_item);
+      const char *text = text_item_get_text (text_item);
+      if (type == TEXT_ITEM_TITLE
+          && e->n_groups >= 1 && e->n_groups <= 4)
+        {
+          char *key = xasprintf ("Head%zu", e->n_groups);
+          string_map_replace (&e->heading_vars, key, text);
+          free (key);
+        }
+      else if (type == TEXT_ITEM_PAGE_TITLE)
+        string_map_replace (&e->heading_vars, "PageTitle", text);
     }
 
-  flush_deferred_syntax (e);
   output_submit__ (e, item);
 }
 
+/* Returns the name of the command currently being parsed, in all uppercase.
+   The caller must free the returned value.
+
+   Returns NULL if no command is being parsed. */
+char *
+output_get_command_name (void)
+{
+  struct output_engine *e = engine_stack_top ();
+  if (e == NULL)
+    return NULL;
+
+  for (size_t i = e->n_groups; i-- > 0;)
+    if (e->groups[i])
+      return utf8_to_upper (e->groups[i]);
+
+  return NULL;
+}
+
 /* Flushes output to screen devices, so that the user can see
    output that doesn't fill up an entire page. */
 void
@@ -181,7 +304,7 @@ output_flush (void)
   struct output_engine *e = engine_stack_top ();
   struct llx *llx;
 
-  flush_deferred_syntax (e);
+  flush_deferred_text (e);
   for (llx = llx_head (&e->drivers); llx != llx_null (&e->drivers);
        llx = llx_next (llx))
     {
@@ -190,6 +313,53 @@ output_flush (void)
         d->class->flush (d);
     }
 }
+
+static void
+output_set_title__ (struct output_engine *e, char **dst, const char *src)
+{
+  free (*dst);
+  *dst = src ? xstrdup (src) : NULL;
+
+  char *page_title
+    = (e->title && e->subtitle ? xasprintf ("%s\n%s", e->title, e->subtitle)
+       : e->title ? xstrdup (e->title)
+       : e->subtitle ? xstrdup (e->subtitle)
+       : xzalloc (1));
+  text_item_submit (text_item_create_nocopy (TEXT_ITEM_PAGE_TITLE,
+                                             page_title));
+}
+
+void
+output_set_title (const char *title)
+{
+  struct output_engine *e = engine_stack_top ();
+
+  output_set_title__ (e, &e->title, title);
+}
+
+void
+output_set_subtitle (const char *subtitle)
+{
+  struct output_engine *e = engine_stack_top ();
+
+  output_set_title__ (e, &e->subtitle, subtitle);
+}
+
+void
+output_set_filename (const char *filename)
+{
+  struct output_engine *e = engine_stack_top ();
+
+  string_map_replace (&e->heading_vars, "Filename", filename);
+}
+
+size_t
+output_get_group_level (void)
+{
+  struct output_engine *e = engine_stack_top ();
+
+  return e->n_groups;
+}
 
 void
 output_driver_init (struct output_driver *driver,
@@ -226,9 +396,11 @@ output_driver_get_engine (const struct output_driver *driver)
 {
   struct output_engine *e;
 
-  for (e = engine_stack; e < &engine_stack[n_stack]; e++)
-    if (llx_find (llx_head (&e->drivers), llx_null (&e->drivers), driver))
-      return e;
+  ll_for_each (e, struct output_engine, ll, &engine_stack)
+    {
+      if (llx_find (llx_head (&e->drivers), llx_null (&e->drivers), driver))
+	return e;
+    }
 
   return NULL;
 }
@@ -258,36 +430,12 @@ output_driver_is_registered (const struct output_driver *driver)
   return output_driver_get_engine (driver) != NULL;
 }
 
-/* Useful functions for output driver implementation. */
-
-void
-output_driver_track_current_command (const struct output_item *output_item,
-                                     char **command_namep)
-{
-  if (is_text_item (output_item))
-    {
-      const struct text_item *item = to_text_item (output_item);
-      const char *text = text_item_get_text (item);
-      enum text_item_type type = text_item_get_type (item);
-
-      if (type == TEXT_ITEM_COMMAND_OPEN)
-        {
-          free (*command_namep);
-          *command_namep = xstrdup (text);
-        }
-      else if (type == TEXT_ITEM_COMMAND_CLOSE)
-        {
-          free (*command_namep);
-          *command_namep = NULL;
-        }
-    }
-}
-
 extern const struct output_driver_factory txt_driver_factory;
 extern const struct output_driver_factory list_driver_factory;
 extern const struct output_driver_factory html_driver_factory;
 extern const struct output_driver_factory csv_driver_factory;
 extern const struct output_driver_factory odt_driver_factory;
+extern const struct output_driver_factory spv_driver_factory;
 #ifdef HAVE_CAIRO
 extern const struct output_driver_factory pdf_driver_factory;
 extern const struct output_driver_factory ps_driver_factory;
@@ -301,6 +449,7 @@ static const struct output_driver_factory *factories[] =
     &html_driver_factory,
     &csv_driver_factory,
     &odt_driver_factory,
+    &spv_driver_factory,
 #ifdef HAVE_CAIRO
     &pdf_driver_factory,
     &ps_driver_factory,
@@ -394,3 +543,85 @@ output_driver_create (struct string_map *options)
 
   return driver;
 }
+
+void
+output_driver_parse_option (const char *option, struct string_map *options)
+{
+  const char *equals = strchr (option, '=');
+  if (equals == NULL)
+    {
+      error (0, 0, _("%s: output option missing `='"), option);
+      return;
+    }
+
+  char *key = xmemdup0 (option, equals - option);
+  if (string_map_contains (options, key))
+    {
+      error (0, 0, _("%s: output option specified more than once"), key);
+      free (key);
+      return;
+    }
+
+  char *value = xmemdup0 (equals + 1, strlen (equals + 1));
+  string_map_insert_nocopy (options, key, value);
+}
+
+/* Extracts the actual text content from the given Pango MARKUP and returns it
+   as as a malloc()'d string. */
+char *
+output_get_text_from_markup (const char *markup)
+{
+  xmlParserCtxt *parser = xmlCreatePushParserCtxt (NULL, NULL, NULL, 0, NULL);
+  if (!parser)
+    return xstrdup (markup);
+
+  xmlParseChunk (parser, "<xml>", strlen ("<xml>"), false);
+  xmlParseChunk (parser, markup, strlen (markup), false);
+  xmlParseChunk (parser, "</xml>", strlen ("</xml>"), true);
+
+  char *content
+    = (parser->wellFormed
+       ? CHAR_CAST (char *,
+                    xmlNodeGetContent (xmlDocGetRootElement (parser->myDoc)))
+       : xstrdup (markup));
+  xmlFreeDoc (parser->myDoc);
+  xmlFreeParserCtxt (parser);
+
+  return content;
+}
+
+char *
+output_driver_substitute_heading_vars (const char *src, int page_number)
+{
+  struct output_engine *e = engine_stack_top ();
+  struct string dst = DS_EMPTY_INITIALIZER;
+  ds_extend (&dst, strlen (src));
+  for (const char *p = src; *p;)
+    {
+      if (!strncmp (p, "&amp;[", 6))
+        {
+          if (page_number != INT_MIN)
+            {
+              const char *start = p + 6;
+              const char *end = strchr (start, ']');
+              if (end)
+                {
+                  const char *value = string_map_find__ (&e->heading_vars,
+                                                         start, end - start);
+                  if (value)
+                    ds_put_cstr (&dst, value);
+                  else if (ss_equals (ss_buffer (start, end - start),
+                                      ss_cstr ("Page")))
+                    ds_put_format (&dst, "%d", page_number);
+                  p = end + 1;
+                  continue;
+                }
+            }
+          ds_put_cstr (&dst, "&amp;");
+          p += 5;
+        }
+      else
+        ds_put_byte (&dst, *p++);
+    }
+  return ds_steal_cstr (&dst);
+}
