@@ -18,6 +18,7 @@
 #define OUTPUT_DRIVER_H 1
 
 #include <stdbool.h>
+#include <stddef.h>
 
 struct output_item;
 struct string_set;
@@ -31,6 +32,16 @@ void output_engine_pop (void);
 void output_submit (struct output_item *);
 void output_flush (void);
 
+void output_set_title (const char *);
+void output_set_subtitle (const char *);
+void output_set_filename (const char *);
+
+char *output_get_command_name (void);
+
+size_t output_get_group_level (void);
+
+void output_driver_parse_option (const char *option,
+                                 struct string_map *options);
 struct output_driver *output_driver_create (struct string_map *options);
 bool output_driver_is_registered (const struct output_driver *);
 
