@@ -1,5 +1,5 @@
 /* Wrapper for <gtk/gtk.h>.
-   Copyright (C) 2011, 2012 Free Software Foundation, Inc.
+   Copyright (C) 2011, 2012, 2017, 2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,21 +18,15 @@
 #define PSPP_GTK_GTK_H
 
 #if __GNUC__ >= 3
-@PRAGMA_SYSTEM_HEADER@
+%PRAGMA_SYSTEM_HEADER%
 #endif
-@PRAGMA_COLUMNS@
+%PRAGMA_COLUMNS%
 
-#@INCLUDE_NEXT@ @NEXT_GTK_GTK_H@
+#%INCLUDE_NEXT% %NEXT_GTK_GTK_H%
 
 
 #ifndef G_CONST_RETURN
 #define G_CONST_RETURN const
 #endif
 
-
-#if (GTK_CHECK_VERSION(3,8,0) == 0)
-#define gtk_widget_register_window(widget, window) gdk_window_set_user_data (window, widget);
-#endif
-
-
 #endif /* PSPP_GTK_GTK_H */
