@@ -1,6 +1,6 @@
 /* PSPPIRE - a graphical user interface for PSPP.
    Copyright (C) 2008, 2009, 2010, 2011, 2012, 2016,
-   2017 Free Software Foundation, Inc.
+   2017, 2019 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,62 +46,35 @@
 #include <gettext.h>
 #define _(msgid) gettext (msgid)
 
-static void psppire_data_editor_class_init          (PsppireDataEditorClass *klass);
-static void psppire_data_editor_init                (PsppireDataEditor      *de);
-
 static void refresh_entry (PsppireDataEditor *);
 
-GType
-psppire_data_editor_get_type (void)
-{
-  static GType de_type = 0;
-
-  if (!de_type)
-    {
-      static const GTypeInfo de_info =
-      {
-	sizeof (PsppireDataEditorClass),
-	NULL, /* base_init */
-        NULL, /* base_finalize */
-	(GClassInitFunc) psppire_data_editor_class_init,
-        NULL, /* class_finalize */
-	NULL, /* class_data */
-        sizeof (PsppireDataEditor),
-	0,
-	(GInstanceInitFunc) psppire_data_editor_init,
-      };
-
-      de_type = g_type_register_static (GTK_TYPE_NOTEBOOK, "PsppireDataEditor",
-					&de_info, 0);
-    }
-
-  return de_type;
-}
+G_DEFINE_TYPE (PsppireDataEditor, psppire_data_editor, GTK_TYPE_NOTEBOOK)
 
 static GObjectClass * parent_class = NULL;
 
+static void
+psppire_data_editor_finalize (GObject *obj)
+{
+  PsppireDataEditor *de = PSPPIRE_DATA_EDITOR (obj);
+  if (de->font)
+    pango_font_description_free (de->font);
+
+  /* Chain up to the parent class */
+  G_OBJECT_CLASS (parent_class)->finalize (obj);
+}
+
 static void
 psppire_data_editor_dispose (GObject *obj)
 {
-  PsppireDataEditor *de = (PsppireDataEditor *) obj;
+  PsppireDataEditor *de = PSPPIRE_DATA_EDITOR (obj);
 
-  if (de->data_store)
-    {
-      g_object_unref (de->data_store);
-      de->data_store = NULL;
-    }
+  if (de->dispose_has_run)
+    return;
 
-  if (de->dict)
-    {
-      g_object_unref (de->dict);
-      de->dict = NULL;
-    }
+  de->dispose_has_run = TRUE;
 
-  if (de->font != NULL)
-    {
-      pango_font_description_free (de->font);
-      de->font = NULL;
-    }
+  g_object_unref (de->data_store);
+  g_object_unref (de->dict);
 
   /* Chain up to the parent class */
   G_OBJECT_CLASS (parent_class)->dispose (obj);
@@ -138,7 +111,7 @@ psppire_data_editor_set_property (GObject         *object,
       g_object_set (de->var_sheet, "split", de->split, NULL);
       break;
     case PROP_DATA_STORE:
-      if ( de->data_store)
+      if (de->data_store)
         {
           g_signal_handlers_disconnect_by_func (de->data_store,
                                                 G_CALLBACK (refresh_entry),
@@ -172,6 +145,7 @@ psppire_data_editor_set_property (GObject         *object,
     case PROP_VALUE_LABELS:
       {
 	gboolean l = g_value_get_boolean (value);
+	de->use_value_labels = l;
 	g_object_set (de->data_sheet, "forward-conversion",
 		      l ?
 		      psppire_data_store_value_to_string_with_labels :
@@ -206,6 +180,7 @@ psppire_data_editor_get_property (GObject         *object,
       g_value_set_pointer (value, de->dict);
       break;
     case PROP_VALUE_LABELS:
+      g_value_set_boolean (value, de->use_value_labels);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -258,6 +233,7 @@ psppire_data_editor_class_init (PsppireDataEditorClass *klass)
   parent_class = g_type_class_peek_parent (klass);
 
   object_class->dispose = psppire_data_editor_dispose;
+  object_class->finalize = psppire_data_editor_finalize;
   object_class->set_property = psppire_data_editor_set_property;
   object_class->get_property = psppire_data_editor_get_property;
 
@@ -269,7 +245,7 @@ psppire_data_editor_class_init (PsppireDataEditorClass *klass)
     g_param_spec_pointer ("data-store",
 			  "Data Store",
 			  "A pointer to the data store associated with this editor",
-			  G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_READABLE );
+			  G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_READABLE);
 
   g_object_class_install_property (object_class,
                                    PROP_DATA_STORE,
@@ -279,7 +255,7 @@ psppire_data_editor_class_init (PsppireDataEditorClass *klass)
     g_param_spec_pointer ("dictionary",
 			  "Dictionary",
 			  "A pointer to the dictionary associated with this editor",
-			  G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_READABLE );
+			  G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_READABLE);
 
   g_object_class_install_property (object_class,
                                    PROP_DICTIONARY,
@@ -437,12 +413,7 @@ static void set_font_recursively (GtkWidget *w, gpointer data);
 void
 psppire_data_editor_data_delete_variables (PsppireDataEditor *de)
 {
-  SswRange *range = SSW_SHEET(de->data_sheet)->selection;
-
-  psppire_dict_delete_variables (de->dict, range->start_x,
-				 (range->end_x - range->start_x + 1));
-
-  gtk_widget_queue_draw (GTK_WIDGET (de->data_sheet));
+  psppire_data_sheet_delete_variables (PSPPIRE_DATA_SHEET (de->data_sheet));
 }
 
 void
@@ -450,6 +421,13 @@ psppire_data_editor_var_delete_variables (PsppireDataEditor *de)
 {
   SswRange *range = SSW_SHEET(de->var_sheet)->selection;
 
+  if (range->start_x > range->end_x)
+    {
+      gint temp = range->start_x;
+      range->start_x = range->end_x;
+      range->end_x = temp;
+    }
+
   psppire_dict_delete_variables (de->dict, range->start_y,
 				 (range->end_y - range->start_y + 1));
 
@@ -469,13 +447,7 @@ psppire_data_editor_insert_new_case_at_posn  (PsppireDataEditor *de, gint posn)
 void
 psppire_data_editor_insert_new_variable_at_posn (PsppireDataEditor *de, gint posn)
 {
-  g_return_if_fail (posn >= 0);
-  const struct variable *v = psppire_dict_insert_variable (de->dict, posn, NULL);
-  g_return_if_fail (v);
-  psppire_data_store_insert_value (de->data_store, var_get_width(v),
-				   var_get_case_index (v));
-
-  gtk_widget_queue_draw (GTK_WIDGET (de));
+  psppire_data_sheet_insert_new_variable_at_posn (PSPPIRE_DATA_SHEET (de->data_sheet), posn);
 }
 
 static void
@@ -487,6 +459,8 @@ psppire_data_editor_init (PsppireDataEditor *de)
   GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (de));
   gtk_style_context_add_class (context, "psppire-data-editor");
 
+  de->dispose_has_run = FALSE;
+
   de->font = NULL;
 
   g_object_set (de, "tab-pos", GTK_POS_BOTTOM, NULL);
@@ -504,6 +478,7 @@ psppire_data_editor_init (PsppireDataEditor *de)
   gtk_box_pack_start (GTK_BOX (hbox), de->datum_entry, TRUE, TRUE, 0);
 
   de->split = FALSE;
+  de->use_value_labels = FALSE;
   de->data_sheet = psppire_data_sheet_new ();
 
   GtkWidget *data_button = ssw_sheet_get_button (SSW_SHEET (de->data_sheet));
@@ -541,7 +516,7 @@ psppire_data_editor_init (PsppireDataEditor *de)
 
   if (psppire_conf_get_string (psppire_conf_new (),
 			   "Data Editor", "font",
-				&fontname) )
+				&fontname))
     {
       de->font = pango_font_description_from_string (fontname);
       g_free (fontname);
@@ -579,10 +554,15 @@ set_font_recursively (GtkWidget *w, gpointer data)
   GtkStyleContext *style = gtk_widget_get_style_context (w);
   GtkCssProvider *cssp = gtk_css_provider_new ();
 
-  gchar *str = pango_font_description_to_string (font_desc);
+  /* The Pango font description as string has a different syntax than the
+     css style description:
+     Pango: Courier Italic 12
+     CSS: italic 12pt Courier
+     I ignore Weight, Style and Variant and just take family and size */
+  const gchar *str = pango_font_description_get_family (font_desc);
+  gint size = pango_font_description_get_size (font_desc);
   gchar *css =
-    g_strdup_printf ("* {font: %s}", str);
-  g_free (str);
+    g_strdup_printf ("* {font: %dpt %s}", size/PANGO_SCALE, str);
 
   GError *err = NULL;
   gtk_css_provider_load_from_data (cssp, css, -1, &err);
@@ -599,7 +579,7 @@ set_font_recursively (GtkWidget *w, gpointer data)
   g_object_unref (cssp);
 
 
-  if ( GTK_IS_CONTAINER (w))
+  if (GTK_IS_CONTAINER (w))
     gtk_container_foreach (GTK_CONTAINER (w), set_font_recursively, font_desc);
 }
 
