@@ -75,7 +75,7 @@ struct _PsppireDialogActionIndepSamps
   GtkWidget *options_button;
 
   /* The variable which determines to which group a datum belongs */
-  const struct variable *grp_var;
+  struct variable *grp_var;
   int grp_var_width;
 
   /* The GtkEntry which holds the reference to the above variable */
