@@ -61,6 +61,8 @@ struct _PsppireDict
   GObject             parent;
   struct dictionary   *dict;
 
+  gboolean dispose_has_run;
+
   gboolean disable_insert_signal;
   /* For GtkTreeModelIface */
   gint stamp;
@@ -119,10 +121,6 @@ void psppire_dict_replace_dictionary (PsppireDict *, struct dictionary *);
 
 struct variable * psppire_dict_get_weight_variable (const PsppireDict *);
 
-#if DEBUGGING
-void psppire_dict_dump (const PsppireDict *);
-#endif
-
 const gchar *psppire_dict_encoding (const PsppireDict *);
 
 G_END_DECLS
