@@ -31,39 +31,10 @@
 #define _(msgid) gettext (msgid)
 #define N_(msgid) msgid
 
-static void psppire_dict_view_base_finalize (PsppireDictViewClass *, gpointer);
-static void psppire_dict_view_base_init     (PsppireDictViewClass *class);
 static void psppire_dict_view_class_init    (PsppireDictViewClass *class);
 static void psppire_dict_view_init          (PsppireDictView      *dict_view);
 
-GType
-psppire_dict_view_get_type (void)
-{
-  static GType psppire_dict_view_type = 0;
-
-  if (!psppire_dict_view_type)
-    {
-      static const GTypeInfo psppire_dict_view_info =
-      {
-	sizeof (PsppireDictViewClass),
-	(GBaseInitFunc) psppire_dict_view_base_init,
-        (GBaseFinalizeFunc) psppire_dict_view_base_finalize,
-	(GClassInitFunc)psppire_dict_view_class_init,
-	(GClassFinalizeFunc) NULL,
-	NULL,
-        sizeof (PsppireDictView),
-	0,
-	(GInstanceInitFunc) psppire_dict_view_init,
-      };
-
-      psppire_dict_view_type =
-	g_type_register_static (GTK_TYPE_TREE_VIEW, "PsppireDictView",
-				&psppire_dict_view_info, 0);
-    }
-
-  return psppire_dict_view_type;
-}
-
+G_DEFINE_TYPE (PsppireDictView, psppire_dict_view, GTK_TYPE_TREE_VIEW)
 
 static void
 psppire_dict_view_finalize (GObject *object)
@@ -290,6 +261,7 @@ psppire_dict_view_get_property (GObject         *object,
 
 	g_value_set_enum (value, gtk_tree_selection_get_mode (selection));
       }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -303,6 +275,8 @@ psppire_dict_view_class_init (PsppireDictViewClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  object_class->finalize = psppire_dict_view_finalize;
+
   GParamSpec *predicate_spec =
     g_param_spec_pointer ("predicate",
 			  "Predicate",
@@ -334,24 +308,6 @@ psppire_dict_view_class_init (PsppireDictViewClass *class)
                                    selection_mode_spec);
 }
 
-
-static void
-psppire_dict_view_base_init (PsppireDictViewClass *class)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-
-  object_class->finalize = psppire_dict_view_finalize;
-}
-
-
-
-static void
-psppire_dict_view_base_finalize (PsppireDictViewClass *class,
-				 gpointer class_data)
-{
-
-}
-
 static gboolean
 use_labels (PsppireDictView *dv)
 {
@@ -700,13 +656,6 @@ psppire_dict_view_init (PsppireDictView *dict_view)
 		    G_CALLBACK (show_menu), NULL);
 }
 
-
-GtkWidget*
-psppire_dict_view_new (void)
-{
-  return GTK_WIDGET (g_object_new (psppire_dict_view_get_type (), NULL));
-}
-
 static struct variable *
 psppire_dict_view_iter_to_var (PsppireDictView *dict_view,
                                GtkTreeIter *top_iter)
