@@ -45,8 +45,6 @@
 #define _(msgid) gettext (msgid)
 #define N_(msgid) msgid
 
-static void psppire_syntax_window_base_finalize (PsppireSyntaxWindowClass *, gpointer);
-static void psppire_syntax_window_base_init     (PsppireSyntaxWindowClass *class);
 static void psppire_syntax_window_class_init    (PsppireSyntaxWindowClass *class);
 static void psppire_syntax_window_init          (PsppireSyntaxWindow      *syntax_editor);
 
@@ -101,44 +99,8 @@ psppire_syntax_window_get_property (GObject         *object,
     };
 }
 
-GType
-psppire_syntax_window_get_type (void)
-{
-  static GType psppire_syntax_window_type = 0;
-
-  if (!psppire_syntax_window_type)
-    {
-      static const GTypeInfo psppire_syntax_window_info =
-      {
-	sizeof (PsppireSyntaxWindowClass),
-	(GBaseInitFunc) psppire_syntax_window_base_init,
-        (GBaseFinalizeFunc) psppire_syntax_window_base_finalize,
-	(GClassInitFunc)psppire_syntax_window_class_init,
-	(GClassFinalizeFunc) NULL,
-	NULL,
-        sizeof (PsppireSyntaxWindow),
-	0,
-	(GInstanceInitFunc) psppire_syntax_window_init,
-      };
-
-      static const GInterfaceInfo window_interface_info =
-	{
-	  (GInterfaceInitFunc) psppire_syntax_window_iface_init,
-	  NULL,
-	  NULL
-	};
-
-      psppire_syntax_window_type =
-	g_type_register_static (PSPPIRE_TYPE_WINDOW, "PsppireSyntaxWindow",
-				&psppire_syntax_window_info, 0);
-
-      g_type_add_interface_static (psppire_syntax_window_type,
-				   PSPPIRE_TYPE_WINDOW_MODEL,
-				   &window_interface_info);
-    }
-
-  return psppire_syntax_window_type;
-}
+G_DEFINE_TYPE_WITH_CODE (PsppireSyntaxWindow, psppire_syntax_window, PSPPIRE_TYPE_WINDOW,
+                         G_IMPLEMENT_INTERFACE (PSPPIRE_TYPE_WINDOW_MODEL, psppire_syntax_window_iface_init))
 
 static GObjectClass *parent_class ;
 
@@ -153,7 +115,7 @@ psppire_syntax_window_finalize (GObject *object)
 static void
 psppire_syntax_window_dispose (GObject *obj)
 {
-  PsppireSyntaxWindow *sw = (PsppireSyntaxWindow *)obj;
+  PsppireSyntaxWindow *sw = PSPPIRE_SYNTAX_WINDOW (obj);
 
   GtkClipboard *clip_selection;
   GtkClipboard *clip_primary;
@@ -187,6 +149,7 @@ psppire_syntax_window_class_init (PsppireSyntaxWindowClass *class)
 {
   GParamSpec *encoding_spec;
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  gobject_class->finalize = psppire_syntax_window_finalize;
 
   GtkSourceLanguageManager *lm = gtk_source_language_manager_get_default ();
 
@@ -229,22 +192,6 @@ psppire_syntax_window_class_init (PsppireSyntaxWindowClass *class)
 }
 
 
-static void
-psppire_syntax_window_base_init (PsppireSyntaxWindowClass *class)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-  object_class->finalize = psppire_syntax_window_finalize;
-}
-
-
-
-static void
-psppire_syntax_window_base_finalize (PsppireSyntaxWindowClass *class,
-				     gpointer class_data)
-{
-}
-
-
 static void
 editor_execute_syntax (const PsppireSyntaxWindow *sw, GtkTextIter start,
 		       GtkTextIter stop)
@@ -391,17 +338,22 @@ clipboard_clear_cb (GtkClipboard *clipboard,
   sw->cliptext = NULL;
 }
 
+static gchar tn1[] = "UTF8_STRING";
+static gchar tn2[] = "STRING";
+static gchar tn3[] = "TEXT";
+static gchar tn4[] = "COMPOUND_TEXT";
+static gchar tn5[] = "text/plain;charset=utf-8";
+static gchar tn6[] = "text/plain";
 
 static const GtkTargetEntry targets[] = {
-  { "UTF8_STRING",   0, SELECT_FMT_TEXT },
-  { "STRING",        0, SELECT_FMT_TEXT },
-  { "TEXT",          0, SELECT_FMT_TEXT },
-  { "COMPOUND_TEXT", 0, SELECT_FMT_TEXT },
-  { "text/plain;charset=utf-8", 0, SELECT_FMT_TEXT },
-  { "text/plain",    0, SELECT_FMT_TEXT },
+  { tn1, 0, SELECT_FMT_TEXT },
+  { tn2, 0, SELECT_FMT_TEXT },
+  { tn3, 0, SELECT_FMT_TEXT },
+  { tn4, 0, SELECT_FMT_TEXT },
+  { tn5, 0, SELECT_FMT_TEXT },
+  { tn6, 0, SELECT_FMT_TEXT }
 };
 
-
 /*
   Store a clip containing the currently selected text.
   Returns true iff something was set.
@@ -1053,7 +1005,7 @@ error_dialog (GtkWindow *w, const gchar *filename,  GError *err)
 /*
   Loads the buffer from the file called FILENAME
 */
-gboolean
+static gboolean
 syntax_load (PsppireWindow *window, const gchar *filename,
              const gchar *encoding, gpointer not_used)
 {
