@@ -44,41 +44,12 @@
 #include "psppire-syntax-window.h"
 #include "psppire-window-register.h"
 
-static void psppire_window_base_init     (PsppireWindowClass *class);
 static void psppire_window_class_init    (PsppireWindowClass *class);
 static void psppire_window_init          (PsppireWindow      *window);
 
-
 static GObjectClass *parent_class;
 
-GType
-psppire_window_get_type (void)
-{
-  static GType psppire_window_type = 0;
-
-  if (!psppire_window_type)
-    {
-      static const GTypeInfo psppire_window_info =
-      {
-	sizeof (PsppireWindowClass),
-	(GBaseInitFunc) psppire_window_base_init,
-        (GBaseFinalizeFunc) NULL,
-	(GClassInitFunc) psppire_window_class_init,
-	(GClassFinalizeFunc) NULL,
-	NULL,
-        sizeof (PsppireWindow),
-	0,
-	(GInstanceInitFunc) psppire_window_init,
-      };
-
-      psppire_window_type =
-	g_type_register_static (PSPPIRE_TYPE_WINDOW_BASE, "PsppireWindow",
-				&psppire_window_info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return psppire_window_type;
-}
-
+G_DEFINE_ABSTRACT_TYPE (PsppireWindow, psppire_window, PSPPIRE_TYPE_WINDOW_BASE)
 
 /* Properties */
 enum
@@ -268,6 +239,8 @@ psppire_window_class_init (PsppireWindowClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  object_class->finalize = psppire_window_finalize;
+
   GParamSpec *description_spec =
     null_if_empty_param ("description",
 		       "Description",
@@ -309,16 +282,6 @@ psppire_window_class_init (PsppireWindowClass *class)
 }
 
 
-static void
-psppire_window_base_init (PsppireWindowClass *class)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-
-  object_class->finalize = psppire_window_finalize;
-}
-
-
-
 static void
 insert_menuitem_into_menu (PsppireWindow *window, gpointer key)
 {
@@ -565,7 +528,8 @@ psppire_window_model_get_type (void)
 	NULL,		/* class_data */
 	0,
 	0,              /* n_preallocs */
-	NULL
+	NULL,
+	NULL            /* value_table */
       };
 
       window_model_type =
@@ -944,4 +908,3 @@ delete_recent (const char *file_name)
 
   g_free (uri);
 }
-
