@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-    Copyright (C) 2005, 2006, 2010, 2011, 2012, 2015  Free Software Foundation
+   Copyright (C) 2005, 2006, 2010, 2011, 2012, 2015, 2020  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,10 @@
 #include "ui/gui/psppire-format.h"
 #include "ui/gui/var-type-dialog.h"
 
+#include <gettext.h>
+#define _(msgid) gettext (msgid)
+#define N_(msgid) msgid
+
 static const struct fmt_spec date_format[] =
   {
     {FMT_DATE,  11, 0},
@@ -51,7 +55,9 @@ static const struct fmt_spec date_format[] =
     {FMT_WKYR, 10, 0},
     {FMT_WKYR, 8, 0},
     {FMT_DATETIME, 17, 0},
-    {FMT_DATETIME, 20, 0}
+    {FMT_DATETIME, 20, 0},
+    {FMT_YMDHMS, 16, 0},
+    {FMT_YMDHMS, 20, 0}
   };
 
 
@@ -201,7 +207,7 @@ psppire_var_type_dialog_new (const struct fmt_spec *format)
                   NULL));
 }
 
-void
+gint
 psppire_var_type_dialog_run (GtkWindow *parent_window,
                              struct fmt_spec *format)
 {
@@ -212,10 +218,13 @@ psppire_var_type_dialog_run (GtkWindow *parent_window,
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
   gtk_widget_show (GTK_WIDGET (dialog));
 
-  if (psppire_dialog_run (PSPPIRE_DIALOG (dialog)) == GTK_RESPONSE_OK)
+  gint result = psppire_dialog_run (PSPPIRE_DIALOG (dialog));
+  if (result == GTK_RESPONSE_OK)
     *format = *psppire_var_type_dialog_get_format (dialog);
 
   gtk_widget_destroy (GTK_WIDGET (dialog));
+
+  return result;
 }
 
 
@@ -225,7 +234,7 @@ on_toggle (GtkToggleButton *togglebutton, gpointer dialog_)
 {
   PsppireVarTypeDialog *dialog = dialog_;
 
-  if ( gtk_toggle_button_get_active (togglebutton) == TRUE)
+  if (gtk_toggle_button_get_active (togglebutton) == TRUE)
     refresh_active_button (dialog);
 }
 
@@ -414,7 +423,7 @@ preview_custom (GtkWidget *w, gpointer data)
 
   PsppireVarTypeDialog *dialog = data;
 
-  if ( dialog->active_button != BUTTON_CUSTOM )
+  if (dialog->active_button != BUTTON_CUSTOM)
     return;
 
   text = gtk_entry_get_text (GTK_ENTRY (dialog->entry_decimals));
@@ -424,7 +433,7 @@ preview_custom (GtkWidget *w, gpointer data)
   dialog->fmt_l.w = atoi (text);
 
   msg_disable ();
-  if ( ! fmt_check_output (&dialog->fmt_l))
+  if (! fmt_check_output (&dialog->fmt_l))
     {
       gtk_label_set_text (GTK_LABEL (dialog->label_psample), "---");
       gtk_label_set_text (GTK_LABEL (dialog->label_nsample), "---");
@@ -528,6 +537,9 @@ psppire_var_type_dialog_constructor (GType                  type,
     type, n_properties, properties);
   dialog = PSPPIRE_VAR_TYPE_DIALOG (obj);
 
+  g_object_set (dialog, "help-page", "Input-and-Output-Formats",
+		"title", _("Variable Type and Format"), NULL);
+
   xml = builder_new ("var-type-dialog.ui");
 
   content_area = GTK_CONTAINER (PSPPIRE_DIALOG (dialog));
@@ -600,7 +612,7 @@ psppire_var_type_dialog_constructor (GType                  type,
 			add_to_group, sizeGroup);
 
 
-  for (i = 0 ; i < num_BUTTONS; ++i )
+  for (i = 0 ; i < num_BUTTONS; ++i)
     g_signal_connect (dialog->radioButton[i], "toggled",
                       G_CALLBACK (on_toggle), dialog);
 
@@ -622,7 +634,7 @@ psppire_var_type_dialog_constructor (GType                  type,
 
   list_store = gtk_list_store_new (1, G_TYPE_STRING);
 
-  for ( i = 0 ; i < sizeof (date_format) / sizeof (date_format[0]) ; ++i )
+  for (i = 0 ; i < sizeof (date_format) / sizeof (date_format[0]) ; ++i)
     {
       const struct fmt_spec *f = &date_format[i];
       gtk_list_store_append (list_store, &iter);
@@ -656,7 +668,7 @@ psppire_var_type_dialog_constructor (GType                  type,
 
   list_store = gtk_list_store_new (1, G_TYPE_STRING);
 
-  for ( i = 0 ; i < sizeof (dollar_format)/sizeof (dollar_format[0]) ; ++i )
+  for (i = 0 ; i < sizeof (dollar_format)/sizeof (dollar_format[0]) ; ++i)
     {
       char *template = settings_dollar_template (&dollar_format[i]);
       gtk_list_store_append (list_store, &iter);
@@ -696,7 +708,7 @@ psppire_var_type_dialog_constructor (GType                  type,
 
   list_store = gtk_list_store_new (1, G_TYPE_STRING);
 
-  for ( i = 0 ; i < 5 ; ++i )
+  for (i = 0 ; i < 5 ; ++i)
     {
       enum fmt_type cc_fmts[5] = {FMT_CCA, FMT_CCB, FMT_CCC, FMT_CCD, FMT_CCE};
       gtk_list_store_append (list_store, &iter);
@@ -746,7 +758,7 @@ psppire_var_type_dialog_constructor (GType                  type,
 
 
 /* Set a particular button to be active */
-void
+static void
 var_type_dialog_set_active_button (PsppireVarTypeDialog *dialog, gint b)
 {
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->radioButton[b]),
@@ -826,6 +838,8 @@ psppire_var_type_dialog_set_state (PsppireVarTypeDialog *dialog)
     case FMT_MOYR:
     case FMT_WKYR:
     case FMT_DATETIME:
+    case FMT_YMDHMS:
+    case FMT_MTIME:
     case FMT_TIME:
     case FMT_DTIME:
     case FMT_WKDAY:
