@@ -33,7 +33,6 @@
 #include "data/dataset.h"
 #include "data/dictionary.h"
 #include "data/file-handle-def.h"
-#include "data/file-name.h"
 #include "data/session.h"
 #include "data/settings.h"
 #include "data/variable.h"
@@ -52,7 +51,6 @@
 #include "ui/source-init-opts.h"
 #include "ui/terminal/terminal-opts.h"
 #include "ui/terminal/terminal-reader.h"
-#include "ui/terminal/terminal.h"
 
 #include "gl/fatal-signal.h"
 #include "gl/progname.h"
@@ -93,7 +91,6 @@ main (int argc, char **argv)
   output_engine_push ();
   fh_init ();
   settings_init ();
-  terminal_check_size ();
   random_init ();
 
   lexer = lex_create ();
@@ -161,13 +158,13 @@ main (int argc, char **argv)
     }
 
 
+  output_engine_pop ();
   session_destroy (the_session);
 
   random_done ();
   settings_done ();
   fh_done ();
   lex_destroy (lexer);
-  output_engine_pop ();
   i18n_done ();
 
   return msg_ui_any_errors ();
@@ -227,7 +224,11 @@ output_msg (const struct msg *m_, void *lexer_)
       m.last_line = lex_get_last_line_number (lexer, 0);
     }
 
+  m.command_name = output_get_command_name ();
+
   message_item_submit (message_item_create (&m));
+
+  free (m.command_name);
 }
 
 static void
