@@ -73,8 +73,8 @@ main (int argc, char *argv[])
   /* Read from stdin into 'input'.  Ensure that 'input' ends in a new-line
      followed by a null byte. */
   input = (!strcmp (file_name, "-")
-           ? fread_file (stdin, &length)
-           : read_file (file_name, &length));
+           ? fread_file (stdin, 0, &length)
+           : read_file (file_name, 0, &length));
   if (input == NULL)
     error (EXIT_FAILURE, errno, "reading %s failed", file_name);
 
@@ -197,7 +197,7 @@ check_segmentation (const char *input, size_t length, bool print_segments)
 
           for (i = MIN (15, strlen (type_name)); i < 16; i++)
             putchar (' ');
-          for (i = 0; i < n; )
+          for (i = 0; i < n;)
             {
               const uint8_t *u_input = CHAR_CAST (const uint8_t *, input);
               ucs4_t uc;
