@@ -38,7 +38,6 @@
 #include <libpspp/compiler.h>
 #include <libpspp/pool.h>
 
-#include "xalloc.h"
 
 /* Exit with a failure code.
    (Place a breakpoint on this function while debugging.) */
@@ -129,7 +128,7 @@ prev_region (unsigned int pattern, unsigned int offset,
   unsigned int i;
 
   assert (offset <= UINT_BIT);
-  for (i = offset; i-- > 0; )
+  for (i = offset; i-- > 0;)
     if (pattern & (1u << i))
       {
         *start = i;
@@ -279,12 +278,6 @@ test_insert (void)
   unsigned int init_pat;
   int i, j;
 
-#if __GNUC__ == 4 && __GNUC_MINOR__ == 2 && __llvm__
-  /* This test seems to trigger a bug in llvm-gcc 4.2 on Mac OS X 10.8.0.
-     Exit code 77 tells the Autotest framework that the test was skipped. */
-  exit (77);
-#endif
-
   for (init_pat = 0; init_pat < (1u << positions); init_pat++)
     for (i = 0; i < positions + 1; i++)
       for (j = i; j <= positions + 1; j++)
@@ -312,12 +305,6 @@ test_delete (void)
   unsigned int init_pat;
   int i, j;
 
-#if __GNUC__ == 4 && __GNUC_MINOR__ == 2 && __llvm__
-  /* This test seems to trigger a bug in llvm-gcc 4.2 on Mac OS X 10.8.0.
-     Exit code 77 tells the Autotest framework that the test was skipped. */
-  exit (77);
-#endif
-
   for (init_pat = 0; init_pat < (1u << positions); init_pat++)
     for (i = 0; i < positions + 1; i++)
       for (j = i; j <= positions + 1; j++)
@@ -342,12 +329,6 @@ test_allocate (void)
   unsigned int init_pat;
   int request;
 
-#if __GNUC__ == 4 && __GNUC_MINOR__ == 2 && __llvm__
-  /* This test seems to trigger a bug in llvm-gcc 4.2 on Mac OS X 10.8.0.
-     Exit code 77 tells the Autotest framework that the test was skipped. */
-  exit (77);
-#endif
-
   for (init_pat = 0; init_pat < (1u << positions); init_pat++)
     for (request = 1; request <= positions + 1; request++)
       {
@@ -398,12 +379,6 @@ test_allocate_fully (void)
   unsigned int init_pat;
   int request;
 
-#if __GNUC__ == 4 && __GNUC_MINOR__ == 2 && __llvm__
-  /* This test seems to trigger a bug in llvm-gcc 4.2 on Mac OS X 10.8.0.
-     Exit code 77 tells the Autotest framework that the test was skipped. */
-  exit (77);
-#endif
-
   for (init_pat = 0; init_pat < (1u << positions); init_pat++)
     for (request = 1; request <= positions + 1; request++)
       {
