@@ -130,7 +130,7 @@ prev_region (unsigned int pattern, unsigned int offset,
   unsigned int i;
 
   assert (offset <= UINT_BIT);
-  for (i = offset; i-- > 0; )
+  for (i = offset; i-- > 0;)
     if (pattern & (1u << i))
       {
         *start = i;
@@ -165,7 +165,7 @@ print_structure (const struct abt_node *node_)
 
   if (node_ == NULL)
     return;
-  node = abt_data (node_, struct range_tower_node, abt_node);
+  node = ABT_DATA (node_, struct range_tower_node, abt_node);
   printf ("%lu+%lu/%d", node->n_zeros, node->n_ones, node->abt_node.level);
   if (node->abt_node.down[0] || node->abt_node.down[1])
     {
